' This is a script to Move clients on SEPM based on the IP address, hostname or user name of the client.
' Created by: Elisha Riedlinger
' Last Modified: 5/02/2011
' Version 3.06

Option Explicit

'--------------------------------------------------------
'Set the below values to your database password and username
'--------------------------------------------------------

Dim DBUser : DBUser = "dba"
Dim DBPass : DBPass = "symantec"
Dim TempGroupName : TempGroupName = "Default Group"

'--------------------------------------------------------
'Other settings
'--------------------------------------------------------

Dim DomainName : DomainName = "%"
Dim OnlyCheckClientsInTempGroup : OnlyCheckClientsInTempGroup = False
Dim SilentMode : SilentMode = False
Dim EnableLogging : EnableLogging = True
Dim sLogFile : sLogFile = "MoveClients.log"
Dim ConnectionString : ConnectionString = "DSN=SymantecEndpointSecurityDSN;UID=" & DBUser & ";PWD=" & DBPass

'--------------------------------------------------------
Dim LogDBPassword : LogDBPassword = False
Dim LogOldClientGroup : LogOldClientGroup = True
Dim UpdateUOClients : UpdateUOClients = False
Dim UseFutureTime : UseFutureTime = False
Dim UseVBStoSearch : UseVBStoSearch = False
Dim ExactMatch : ExactMatch = False
'--------------------------------------------------------
'--------------------------------------------------------
'--------------------------------------------------------
'--------------------------------------------------------

'Checking for 64-bit Windows
Dim Wshshell : Set WshShell = WScript.CreateObject("WScript.Shell")
Dim ProcessorType : ProcessorType = WshShell.RegRead("HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0\Identifier")
If NOT (InStr(ProcessorType, "x86") > 0) Then
	' MsgBox("64-bit Windows OS detected.")
	' Checking if the script is running in 32-bit mode already
	If NOT (InStr(ucase(WScript.FullName), "SYSWOW64") > 0) Then
		' ReRun script
		Wshshell.Run "%windir%\SysWOW64\wscript.exe """ & WScript.ScriptFullName & """"
		WScript.Quit(0)
	End If
End If

'Declare variables
Dim RootGroup, DomainID, TempGroupID, GroupStructureXML
Dim GroupList(), ListCount, TotalCount, AllClientNames, AllClientNamesWild
Dim DomainList(), DomainCount, DomainNum
Dim SearchType, OutputFile, FoundClients, ExitMessage
Dim NewTimeStamp, NewUSN, IsOUClient
Dim oFSO, connect, connectUpdate
Dim UpdateFlag : UpdateFlag = False
Dim AnyFailures : AnyFailures = False
Set oFSO = CreateObject("Scripting.FileSystemObject")


If UpdateUOClients then
	IsOUClient = ""
Else
	IsOUClient = " and SEM_CLIENT.GROUP_IS_OU = 0 "
End if
If UseFutureTime then
	NewTimeStamp = "(Select (Cast(DateDiff(second, '1970-01-01', GetDate()) as bigint) + 86400) * 1000)"  'Time stamp for one day in the future
Else
	NewTimeStamp = "(Select Cast(DateDiff(second, '1970-01-01', GetDate()) as bigint) * 1000)"
End if

'---------------------------- Vars for MD5 conversion ---------------------
Private lngTrack
Private arrLongConversion(4)
Private arrSplit64(63)

Private Const OFFSET_4 = 4294967296
Private Const MAXINT_4 = 2147483647

Private Const S11 = 7
Private Const S12 = 12
Private Const S13 = 17
Private Const S14 = 22
Private Const S21 = 5
Private Const S22 = 9
Private Const S23 = 14
Private Const S24 = 20
Private Const S31 = 4
Private Const S32 = 11
Private Const S33 = 16
Private Const S34 = 23
Private Const S41 = 6
Private Const S42 = 10
Private Const S43 = 15
Private Const S44 = 21
'---------------------------- End vars for MD5 conversion ---------------------

'Starting Log Output
call StartLog

'Testing SQL Connection
call SQLConnect

'Get Root Group Name
If GetRootGroupName() = False then
	ShowMessage(ExitMessage)
	call CloseLog()
Else
	DomainCount = 0
	ReDim DomainList(3, 0)
	DomainList(0, 0) = RootGroup
	DomainList(1, 0) = DomainID
	DomainList(2, 0) = lcase(DomainName)
	DomainList(3, 0) = GroupStructureXML
End if

'MoveClients
call StartRun("IPgroups.txt", 1)
call StartRun("Hostgroups.txt", 2)
call StartRun("Usergroups.txt", 3)
call StartRun("OSgroups.txt", 4)
call StartRun("SwitchClientGroups.txt", 5)

'Notify user script completed
ShowMessage("Script Completed" & vbCRLF & vbCRLF & "All activities of this script have been logged to " & sLogFile)

'Close and exit
call CloseLog()
WScript.Quit(0)


'--------------------------------------------------------
Sub StartLog ()
	Const ForAppending = 8

	On Error Resume Next
	Err.Clear      ' Clear the error.

	If EnableLogging then
		If oFSO.FileExists(sLogFile) Then
			Set OutputFile = oFSO.OpenTextFile(sLogFile, ForAppending, True)
		Else
			Set OutputFile = oFSO.CreateTextFile(sLogFile, True)
		End if
	End if

	LogEvent("########################################################################" & vbCRLF & "# Begin MoveClient		" & Date() & " " & Time() & vbCRLF & "########################################################################" & vbCRLF)
	If LogDBPassword then LogEvent("::SQL User for this run: " & DBUser & vbCRLF & "::SQL Pass for this run: " & DBPass & vbCRLF)
	LogEvent("::Temp Group for this run: " & TempGroupName & vbCRLF)
End Sub


Sub SQLConnect()
	Dim sql, sErrorMessage

	On Error Resume Next
	Err.Clear      ' Clear the error.

	If LogDBPassword Then LogEvent(Date() & " " & Time() & " Testing connection to ODBC DSN: " & ConnectionString & vbCRLF)
	Set connect = CreateObject("ADODB.Connection")
	connect.ConnectionTimeout = 60
	connect.CommandTimeout = 360
	connect.CursorLocation = 2  'adUseServer
	connect.Open ConnectionString
	Set connectUpdate = CreateObject("ADODB.Connection")
	connectUpdate.ConnectionTimeout = 60
	connectUpdate.CommandTimeout = 60
	connectUpdate.Open ConnectionString

	If err.number = 0 Then
		LogEvent("		::Success" & vbCRLF)
	Else
		LogEvent("		::Failure. Return Code: " & err.number)
		If err.number = -2147217843 Then
			LogEvent(" - Incorrect Username/Password" & vbCRLF)
			sErrorMessage = "Incorrect Username/Password. Verify the username and password values set in this script."
		ElseIf err.number = -2147467259 Then
			LogEvent(" - Unable to find the ODBC DSN specified" & vbCRLF)
			sErrorMessage = "Unable to open a connection to the SEPM Database. Verify this script is running on the SEPM."
		Else
			LogEvent(" - This error was not experienced during testing of this tool." & vbCRLF)
			sErrorMessage = "Unhandled exception. Please review SEPMMoveClients.log on your root drive for further information."
		End If
		Err.Clear      ' Clear the error.
		ShowMessage("Could not connect to the database using ODBC DSN. Verify the following:" & sErrorMessage & vbCRLF & "This script will now exit.")
		call CloseLog()  
	End If
End Sub


Function GetRootGroupName ()
	Dim sql, resultSet

	'SQL Connection
	On Error Resume Next
	Err.Clear      ' Clear the error.

	GetRootGroupName = True

	sql = "Select IDENTITY_MAP.Name as DomainName, IM.* from IDENTITY_MAP, IDENTITY_MAP as IM where IM.TYPE = 'SemClientGroup' and not IM.name like '%\%' and IDENTITY_MAP.Domain_ID = IM.Domain_ID and IDENTITY_MAP.type = 'SemDomain' and IDENTITY_MAP.name like '" & DomainName & "';"
	'LogEvent("	::Root Group Name Query: " & sql & vbCRLF)

	Set resultSet = connect.Execute(sql)
	If err.number = 0 and NOT resultSet.EOF Then
		RootGroup = resultSet("name")
		DomainID = resultSet("domain_id")
		DomainName = resultSet("DomainName")
		LogEvent("	::Root Group Name: " & RootGroup & vbCRLF & "	::Domain Name: " & DomainName & vbCRLF & "	::DomainID: " & DomainID & vbCRLF & "		::Success" & vbCRLF)
		resultSet.Close
	Else
		LogEvent("	::Root Group where domain name is " & DomainName & vbCRLF & "		::Failed" & vbCRLF)
		ExitMessage = "Failed to query the database for the Root Group Name and Domain ID." & vbCRLF & "This script will now exit."
		Err.Clear      ' Clear the error.
		resultSet.Close
		GetRootGroupName = False
		Exit Function
	End If

	sql = "Select * from IDENTITY_MAP where TYPE = 'SemClientGroup' and name like '" & RootGroup & "\" & TempGroupName & "' and DOMAIN_ID = '" & DomainID & "';"
	'LogEvent("	::Temporary GroupID Query: " & sql & vbCRLF)

	Set resultSet = connect.Execute(sql)
	If err.number = 0 and NOT resultSet.EOF Then
		TempGroupID = resultSet("id")
		resultSet.Close
		LogEvent("	::Get Temp GroupID: " & TempGroupID & vbCRLF & "		::Success" & vbCRLF)
	Else
		LogEvent("	::Get Temp GroupID " & vbCRLF & "		::Failed" & vbCRLF)
		If OnlyCheckClientsInTempGroup then
			ExitMessage = "Failed to query the database for the Temp GroupID." & vbCRLF & "This script will now exit."
			GetRootGroupName = False
			Exit Function
		End if
		Err.Clear      ' Clear the error.
		resultSet.Close
	End If

	sql = "Select SEQ_NUM + 1 as NewUSN from SE_Global;"
	'LogEvent("	::NewUSN Query: " & sql & vbCRLF)

	Set resultSet = connect.Execute(sql)
	If err.number = 0 and NOT resultSet.EOF Then
		NewUSN = resultSet("NewUSN")
		resultSet.Close
		LogEvent("	::Get NewUSN: " & NewUSN & vbCRLF & "		::Success" & vbCRLF)
	Else
		LogEvent("	::Get NewUSN " & vbCRLF & "		::Failed" & vbCRLF)
		If OnlyCheckClientsInTempGroup then
			ExitMessage = "Failed to query the database for the NewUSN." & vbCRLF & "This script will now exit."
			GetRootGroupName = False
			Exit Function
		End if
		Err.Clear      ' Clear the error.
		resultSet.Close
	End If

	LogEvent(Now() & " : Start structure metadata Query" & vbCRLF)
	sql = "Select CONTENT from BASIC_METADATA where TYPE = 'SemClientGroupTree' and DOMAIN_ID = '" & DomainID & "';"
	'LogEvent("	::Group structure metadata Query: " & sql & vbCRLF)

	Set resultSet = connect.Execute(sql)
	Dim BinaryData
	BinaryData = resultSet("CONTENT")
	If IsNull(BinaryData) then
		BinaryData = resultSet("CONTENT").GetChunk(resultSet("CONTENT").ActualSize)
	End if
	If NOT IsNull(BinaryData) then
		GroupStructureXML = lcase("  " & RSBinaryToString(BinaryData) & " ")
	End if

	If (err.number = 0) and NOT IsNull(BinaryData) Then
		resultSet.Close
		LogEvent("	::Get group structure metadata" & vbCRLF & "		::Success" & vbCRLF)
	Else
		LogEvent("	::Get group structure metadata" & vbCRLF & "		::Failed: " & vbCRLF)
		Err.Clear      ' Clear the error.
		resultSet.Close
	End If
End Function


Sub StartRun(sFile, newSearchType)
	Dim oFile, sText

	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	'Init vars
	SearchType = newSearchType
	FoundClients = False
	ListCount = 0
	TotalCount = 1000
	ReDim GroupList(4, TotalCount)
	
	'Open group file
	If oFSO.FileExists(sFile) Then
		Set oFile = oFSO.OpenTextFile(sFile, 1)
		If err.number = 0 Then
			LogEvent("	::Processing " & sFile & vbCRLF)
		Else
			Err.Clear      ' Clear the error.
			LogEvent("Unable to open file: " & sFile & vbCRLF)
			exit sub
		End If
		Dim bFileEmpty : bFileEmpty = TRUE
		Do While Not oFile.AtEndOfStream
			sText = oFile.ReadLine
			If Trim(sText) <> "" Then
				bFileEmpty = FALSE
				call AnalyzeString(sText)
			End If
		Loop
		oFile.Close
		If bFileEmpty = TRUE Then
			LogEvent("Empty settings file: " & sFile & vbCRLF)
			exit sub
		End If
	Else
		LogEvent("Missing settings file: " & sFile & vbCRLF)
		exit sub
	End If

	'Update client groups
	Dim x
	For DomainNum = 0 to DomainCount
		RootGroup = DomainList(0, DomainNum)
		DomainID = DomainList(1, DomainNum)
		DomainName = DomainList(2, DomainNum)
		GroupStructureXML = DomainList(3, DomainNum)
		
		If SearchType = 5 then
			call SwitchClientGroups
		Else
			If UseVBStoSearch = False then
				call UpdateClientGroups
			Else
				call UpdateClientGroupsVBS
			End if
		End if
	Next
	If SearchType = 2 or SearchType = 3 Then
		For x = 1 to ListCount
			If GroupList(4, x) <> True Then LogEvent("		::Could not find client '" & GroupList(2, x) & "' in domain '" & DomainList(2, GroupList(0, x)) & "'." & vbCRLF)
		Next
	End if
End Sub


Function GetDomainList (DomainText)
	Dim x
	GetDomainList = -1
	For x = 0 to DomainCount
		If DomainText = DomainList(2, x) then
			GetDomainList = x
			Exit Function
		End if
	Next
	DomainName = DomainText
	If GetRootGroupName() then
		DomainCount = DomainCount + 1
		GetDomainList = DomainCount
		ReDim Preserve DomainList(3, DomainCount)
		DomainList(0, DomainCount) = RootGroup
		DomainList(1, DomainCount) = DomainID
		DomainList(2, DomainCount) = lcase(DomainName)
		DomainList(3, DomainCount) = GroupStructureXML
	End if
End Function


Sub AnalyzeString (inputString)
	Dim StringPart, HostText, GroupText, SubnetNote, SubnetNoteNum, DomainText
	Dim Dash, Slash, SubnetLowText, SubnetHighText, SubnetLowNum, SubnetHighNum, SubnetText
	Dim ThisDomain
	StringPart = Split(inputString, ",")
	If Ubound(StringPart) = 1 or Ubound(StringPart) = 2 then
		If Ubound(StringPart) = 2 then
			DomainText = lcase(trim(StringPart(1)))
			ThisDomain = GetDomainList(DomainText)
			If ThisDomain = -1 then
				Exit Sub
			End if
			GroupText = trim(StringPart(2))
		Else
			ThisDomain = 0
			GroupText = trim(StringPart(1))
		End if
		If SearchType = 1 then  'This is an IP search type
			SubnetText = replace(StringPart(0), " ", "")
			Dash = instr(SubnetText, "-")
			Slash = instr(SubnetText, "/")
			If Dash > 0 then
				SubnetLowText = mid(SubnetText, 1, Dash - 1)
				SubnetHighText = mid(SubnetText, Dash + 1, len(SubnetText))
				SubnetLowNum = GetSubnet(SubnetLowText)
				SubnetHighNum = GetSubnet(SubnetHighText)
			ElseIf Slash > 0 then
				SubnetLowText = mid(SubnetText, 1, Slash - 1)
				SubnetNote = mid(SubnetText, Slash + 1, len(SubnetText))
				SubnetNoteNum = GetSubnetNote(SubnetNote)
				SubnetLowNum = GetSubnetMasked(GetSubnet(SubnetLowText), SubnetNoteNum, True)
				SubnetHighNum = GetSubnetMasked(SubnetLowNum, SubnetNoteNum, False)
			Else
				SubnetLowNum = GetSubnet(SubnetText)
				SubnetHighNum = SubnetLowNum
			End if
			If SubnetLowNum <> 0 and SubnetHighNum <> 0 and GroupText <> "" then
				ListCount = ListCount + 1
				if ListCount > TotalCount then
					TotalCount = TotalCount + 1000
					ReDim Preserve GroupList(4, TotalCount)
				End if
				GroupList(0, ListCount) = ThisDomain
				GroupList(1, ListCount) = Replace(GroupText, "'", "''")
				GroupList(2, ListCount) = SubnetLowNum
				GroupList(3, ListCount) = SubnetHighNum
				LogEvent("	::" & GroupList(2, ListCount) & " - " & GroupList(3, ListCount) & " -|- " & DomainList(2, ThisDomain) & " -|- " & GroupList(1, ListCount) & vbCRLF)
			End if
		Else
			HostText = trim(StringPart(0))
			If HostText <> "" and GroupText <> "" then
				ListCount = ListCount + 1
				if ListCount > TotalCount then
					TotalCount = TotalCount + 1000
					ReDim Preserve GroupList(4, TotalCount)
				End if
				GroupList(0, ListCount) = ThisDomain
				GroupList(1, ListCount) = Replace(GroupText, "'", "''")
				If (UseVBStoSearch = True) Then
					GroupList(2, ListCount) = Replace(lcase(HostText), "'", "''")
				Else
					GroupList(2, ListCount) = Replace(HostText, "'", "''")
				End if
				LogEvent("	::" & GroupList(2, ListCount) & " -|- " & DomainList(2, ThisDomain) & " -|- " & GroupList(1, ListCount) & vbCRLF)
			End if
		End if
	End if
End Sub


Sub SwitchClientGroups
	Dim sql, resultSet, x, PolicyMode, CheckPolicyMode, Flag
	Dim GroupName, GetPolicyMode, UserName, UserDomain, UserDomainName, Hash

	On Error Resume Next
	Err.Clear      ' Clear the error.

	x = 0
	Do
		x = x + 1
		If GroupList(0, x) = DomainNum then
			GroupName = GroupList(1, x)
			GetPolicyMode = lcase(GroupList(2, x))
			
			'Get policy mode
			PolicyMode = ""
			CheckPolicyMode = ""
			If (InStr(GetPolicyMode, "to") > 0) and (InStr(GetPolicyMode, "user") > 0) then
				PolicyMode = "0"
				CheckPolicyMode = "1"
			ElseIf (InStr(GetPolicyMode, "to") > 0) and (InStr(GetPolicyMode, "computer") > 0) then
				PolicyMode = "1"
				CheckPolicyMode = "0"
			End if
			
			'Check policy mode and start query
			If PolicyMode <> "" and CheckPolicyMode <> "" then
				Err.Clear      ' Clear the error.
				
				LogEvent("	::Checking for clients to switch in group " & RootGroup & "\" & GroupName & vbCRLF)
				sql = "Select SEM_CLIENT.COMPUTER_ID, SEM_CLIENT.CLIENT_ID, SEM_COMPUTER.COMPUTER_NAME, SEM_COMPUTER.COMPUTER_DOMAIN_NAME, SEM_CLIENT.USER_NAME, SEM_CLIENT.USER_DOMAIN_NAME, SEM_CLIENT.HARDWARE_KEY, SEM_COMPUTER.CURRENT_LOGIN_USER, SEM_COMPUTER.CURRENT_LOGIN_DOMAIN "
				sql = sql & "from SEM_CLIENT, SEM_COMPUTER, IDENTITY_MAP where SEM_CLIENT.COMPUTER_ID = SEM_COMPUTER.COMPUTER_ID and SEM_CLIENT.DELETED = 0 and SEM_COMPUTER.DELETED = 0 "
				sql = sql & "and SEM_CLIENT.DOMAIN_ID = '" & DomainID & "' and SEM_CLIENT.POLICY_MODE = " & CheckPolicyMode & " and IDENTITY_MAP.ID = SEM_CLIENT.GROUP_ID " & IsOUClient
				sql = sql & "and IDENTITY_MAP.TYPE = 'SemClientGroup' and IDENTITY_MAP.NAME like '" & RootGroup & "\" & GroupName & "' and IDENTITY_MAP.DOMAIN_ID = '" & DomainID & "';"
				'LogEvent("	::Temp Group client List SQL Query: " & sql & vbCRLF)

				Set resultSet = CreateObject("ADODB.recordset")
				resultSet.CursorLocation = 2  'adUseServer
				resultSet.CursorType = 0  'adOpenForwardOnly
				resultSet.LockType = 1  'adLockReadOnly

				resultSet.Open sql, connect
				If err.number = 0 Then
					
					FoundClients = False
					Do While Not resultSet.EOF
						FoundClients = True
						Flag = True
						
						'Check if the record is not deleted
						If CheckForDeleted(resultSet("CLIENT_ID")) then
							Flag = False
							LogEvent("		::Cannot convert " & resultSet("COMPUTER_NAME") & " to user mode. The record is deleted." & vbCRLF)
						End if
						
						'Get username and domain name and check if they are blank
						If (resultSet("USER_NAME") & "" <> "") and (resultSet("USER_DOMAIN_NAME") & "" <> "") then
							UserName = resultSet("USER_NAME")
							UserDomain = resultSet("USER_DOMAIN_NAME")
						ElseIf (resultSet("CURRENT_LOGIN_USER") & "" <> "") and (resultSet("CURRENT_LOGIN_DOMAIN") & "" <> "") then
							UserName = resultSet("CURRENT_LOGIN_USER")
							UserDomain = resultSet("CURRENT_LOGIN_DOMAIN")
						Else
							If PolicyMode = "0" then
								Flag = False
								LogEvent("		::Cannot convert " & resultSet("COMPUTER_NAME") & " to user mode. Username or domain is blank." & vbCRLF)
							End if
							UserName = ""
							UserDomain = ""
						End if
						
						'Check if user already exists
						'If Flag and PolicyMode = "0" then
						'	If CheckForUser(UserName, UserDomain) = True then
						'		Flag = False
						'		LogEvent("		::Cannot convert " & resultSet("COMPUTER_NAME") & " to user mode. User already exists." & vbCRLF)
						'	End if
						'End if
						
						'Check if computer name is blank
						If Flag and (PolicyMode = "1") and ((resultSet("COMPUTER_NAME") & "" = "") or (resultSet("COMPUTER_DOMAIN_NAME") & "" = "")) then
							Flag = False
							LogEvent("		::Cannot convert " & UserName & " to computer mode. Computer name or domain is blank." & vbCRLF)
						End if
						
						'Get hash
						Hash = ""
						If PolicyMode = "0" then
							UserDomainName = UserName & UserDomain
						Else
							UserDomainName = ""
						End if
						If Flag then Hash = UCase(CalculateMD5(DomainID & PolicyMode & UserDomainName & resultSet("COMPUTER_NAME") & resultSet("COMPUTER_DOMAIN_NAME") & resultSet("HARDWARE_KEY")))
						If Flag and Hash = "" then
							Flag = False
							LogEvent("		::Cannot compute hash for " & resultSet("COMPUTER_NAME") & ".  Cannot convert to user mode." & vbCRLF)
						End if
						
						If Flag then call SwitchClient(resultSet("COMPUTER_ID"), resultSet("CLIENT_ID"), PolicyMode, UserName, UserDomain, resultSet("COMPUTER_NAME") & "", resultSet("COMPUTER_DOMAIN_NAME") & "", resultSet("HARDWARE_KEY") & "", Hash)
						resultSet.MoveNext
					Loop
					If not FoundClients then LogEvent("		::No clients found in " & GroupName & " that need to be switched." & vbCRLF)
				Else
					LogEvent("		::Failed to retrieve client list from database." & vbCRLF)
				End If
			Else
				LogEvent("		::Invalid policy mode for group: " & GroupName & vbCRLF)
			End If
		End If
	Loop until x >= ListCount

	resultSet.Close
End Sub


Function CheckForDeleted (ClientID)
	Dim sql, resultSet
	
	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	sql = "Select DELETED from SEM_CLIENT where CLIENT_ID = '" & ClientID & "' and DOMAIN_ID = '" & DomainID & "';"
	'LogEvent("	::Check for client SQL Query: " & sql & vbCRLF)
	
	Set resultSet = connect.Execute(sql)
	
	If NOT resultSet.EOF then
		If resultSet("DELETED") = "0" then
			CheckForDeleted = False
		Else
			CheckForDeleted = True
		End if
	Else
		CheckForDeleted = True
	End if
	
	resultSet.Close
End Function


Function CheckForUser (UserName, UserDomain)
	Dim sql, resultSet
	
	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	sql = "Select USER_NAME, USER_DOMAIN_NAME "
	sql = sql & "from SEM_CLIENT where USER_NAME = '" & UserName & "' and USER_DOMAIN_NAME = '" & UserDomain & "' "
	sql = sql & "and DOMAIN_ID = '" & DomainID & "' and POLICY_MODE = 0 " & IsOUClient & " and DELETED = 0;"
	'LogEvent("	::Check for client SQL Query: " & sql & vbCRLF)
	
	Set resultSet = connect.Execute(sql)
	
	If NOT resultSet.EOF then
		CheckForUser = True
	Else
		CheckForUser = False
	End if
	
	resultSet.Close
End Function


Sub SwitchClient (ComputerID, ClientID, PolicyMode, UserName, UserDomain, ComputerName, ComputerDomain, HardwareKey, Hash)
	Dim sql, SQLPart, resultSet

	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	FoundClients = True
	If PolicyMode = "0" then
		LogEvent(Date() & " " & Time() & "		::Switching Client " & ComputerName  & " to user mode: " & UserName & vbCRLF)
	Else
		LogEvent(Date() & " " & Time() & "		::Switching Client " & UserName  & " to computer mode: " & ComputerName & vbCRLF)
	End if

	connectUpdate.BeginTrans
	sql = "Update SEM_Computer set CURRENT_LOGIN_USER='" & UserName & "', CURRENT_LOGIN_DOMAIN='" & UserDomain & "', TIME_STAMP=" & NewTimeStamp & ", USN=" & NewUSN & " where COMPUTER_ID='" & ComputerID & "';"
	connectUpdate.Execute sql
	SQLPart = "Select Case When (Select Sum(A.GROUP_IS_OU) + Sum(A.PIN_MARK) from SEM_Client as A, SEM_Client as B where A.POLICY_MODE = 0 and A.USER_NAME = B.USER_NAME and A.USER_DOMAIN_NAME = B.USER_DOMAIN_NAME and B.CLIENT_ID='" & ClientID & "') > 0 then 1 Else 0 End"
	SQLPart = "(Case When (Select Sum(GROUP_IS_OU) + Sum(PIN_MARK) + (" & SQLPart & ") from SEM_Client where CLIENT_ID='" & ClientID & "') > 0 then 1 Else 0 End)"
	sql = "Update SEM_Client set  PIN_MARK=" & SQLPart & ", USER_NAME='" & UserName & "', USER_DOMAIN_NAME='" & UserDomain & "', POLICY_MODE=" & PolicyMode & ", HASH='" & Hash & "', TIME_STAMP=" & NewTimeStamp & ", USN=" & NewUSN & ", GROUP_IS_OU=0 where CLIENT_ID='" & ClientID & "';"
	connectUpdate.Execute sql
	sql = "Update SEM_Client set DELETED=1, TIME_STAMP=" & NewTimeStamp & ", USN=" & NewUSN & " where CLIENT_ID<>'" & ClientID & "' and COMPUTER_ID='" & ComputerID & "' " & IsOUClient & ";"
	connectUpdate.Execute sql
	If err.number <> 0 Then
		LogEvent("				::Failed to switch client error " & err.number & vbCRLF)
		Err.Clear      ' Clear the error.
		connectUpdate.RollbackTrans
		If err.number <> 0 Then LogEvent("				::Failed to rollback transaction " & err.number & vbCRLF)
		AnyFailures = True
	Else
		UpdateFlag = true
		connectUpdate.CommitTrans
		If err.number <> 0 Then
			LogEvent("				::Failed to commit transaction " & err.number & vbCRLF)
			AnyFailures = True
		End if
	End if
End Sub


Sub UpdateClientGroupsVBS
	Dim sql, resultSet, GroupName, x, PolicyMode

	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	AllClientNames = ""
	AllClientNamesWild = False
	If SearchType = 2 or SearchType = 3 Then
		For x = 1 to ListCount
			If GroupList(0, x) = DomainNum then
				AllClientNames = AllClientNames & GroupList(2, x) & " "
			End if
		Next
		If InStr(AllClientNames, "*") > 0 or InStr(AllClientNames, "?") > 0 then AllClientNamesWild = True
	End if

	LogEvent("	::Checking for clients to move" & vbCRLF)
	sql = "Select SEM_CLIENT.COMPUTER_ID, SEM_CLIENT.CLIENT_ID, SEM_COMPUTER.OPERATION_SYSTEM, SEM_COMPUTER.COMPUTER_NAME, SEM_CLIENT.USER_NAME, SEM_CLIENT.GROUP_ID, SEM_CLIENT.POLICY_MODE, "
	'Get IP address 1
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR1 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR1 else SEM_COMPUTER.IP_ADDR1 end / 256 / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR1 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR1 else SEM_COMPUTER.IP_ADDR1 end / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR1 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR1 else SEM_COMPUTER.IP_ADDR1 end / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST( case when SEM_COMPUTER.IP_ADDR1 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR1 else SEM_COMPUTER.IP_ADDR1 end & 0xFF as VARCHAR) as IP_ADDR1_TEXT, "
	'Get IP address 2
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR2 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR2 else SEM_COMPUTER.IP_ADDR2 end / 256 / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR2 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR2 else SEM_COMPUTER.IP_ADDR2 end / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR2 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR2 else SEM_COMPUTER.IP_ADDR2 end / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST( case when SEM_COMPUTER.IP_ADDR2 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR2 else SEM_COMPUTER.IP_ADDR2 end & 0xFF as VARCHAR) as IP_ADDR2_TEXT, "
	'Get IP address 3
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR3 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR3 else SEM_COMPUTER.IP_ADDR3 end / 256 / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR3 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR3 else SEM_COMPUTER.IP_ADDR3 end / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR3 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR3 else SEM_COMPUTER.IP_ADDR3 end / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST( case when SEM_COMPUTER.IP_ADDR3 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR3 else SEM_COMPUTER.IP_ADDR3 end & 0xFF as VARCHAR) as IP_ADDR3_TEXT, "
	'Get IP address 4
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR4 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR4 else SEM_COMPUTER.IP_ADDR4 end / 256 / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR4 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR4 else SEM_COMPUTER.IP_ADDR4 end / 256 / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST((case when SEM_COMPUTER.IP_ADDR4 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR4 else SEM_COMPUTER.IP_ADDR4 end / 256) & 0xFF as VARCHAR) + '.' + "
	sql=sql & "CAST( case when SEM_COMPUTER.IP_ADDR4 < 0 then 0xFFFFFFFF + SEM_COMPUTER.IP_ADDR4 else SEM_COMPUTER.IP_ADDR4 end & 0xFF as VARCHAR) as IP_ADDR4_TEXT "
	'SQL from statement
	sql = sql & "from SEM_CLIENT, SEM_COMPUTER where SEM_CLIENT.COMPUTER_ID = SEM_COMPUTER.COMPUTER_ID " & IsOUClient & " and SEM_CLIENT.DELETED = 0 and SEM_COMPUTER.DELETED = 0 "
	sql = sql & "and SEM_CLIENT.DOMAIN_ID = '" & DomainID & "' "
	If OnlyCheckClientsInTempGroup Then
		sql = sql & "and SEM_CLIENT.group_id = '" & TempGroupID & "' "
	End if
	sql = sql & "Order by SEM_CLIENT.TIME_STAMP DESC;"
	'LogEvent("	::Temp Group client List SQL Query: " & sql & vbCRLF)

	Set resultSet = CreateObject("ADODB.recordset")
	resultSet.CursorLocation = 2  'adUseServer
	resultSet.CursorType = 0  'adOpenForwardOnly
	resultSet.LockType = 1  'adLockReadOnly

	resultSet.Open sql, connect
	If err.number = 0 Then
		Do While Not resultSet.EOF
			GroupName = ""

			PolicyMode = resultSet("POLICY_MODE")
			If SearchType = 1 and PolicyMode = 1  then GroupName = GetGroupNameIP(resultSet("IP_ADDR1_TEXT") & "", resultSet("IP_ADDR2_TEXT") & "", resultSet("IP_ADDR3_TEXT") & "", resultSet("IP_ADDR4_TEXT") & "")
			If SearchType = 2 and PolicyMode = 1  then GroupName = GetGroupName(resultSet("COMPUTER_NAME") & "")
			If SearchType = 3 and PolicyMode = 0 then GroupName = GetGroupName(resultSet("USER_NAME") & "")
			If SearchType = 4 and PolicyMode = 1 then GroupName = GetGroupName(resultSet("OPERATION_SYSTEM") & "")

			If GroupName <> "" then call MoveClient (GroupName, resultSet("COMPUTER_NAME") & "", resultSet("USER_NAME") & "", resultSet("COMPUTER_ID"), resultSet("CLIENT_ID"), resultSet("GROUP_ID"))
			resultSet.MoveNext
		Loop
		If not FoundClients then LogEvent("		::No clients found that need to be moved." & vbCRLF)
	Else
		LogEvent("		::Failed to retrieve client list from database." & vbCRLF)
	End If

	resultSet.Close
End Sub


Sub UpdateClientGroups
	Dim sql, resultSet, GroupName, x, sqlPart1, sqlPart2

	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	LogEvent("	::Checking for clients to move" & vbCRLF)
	For x = 1 to ListCount
		If GroupList(0, x) = DomainNum then
			sql = "Select SEM_COMPUTER.COMPUTER_NAME, SEM_CLIENT.USER_NAME, SEM_CLIENT.COMPUTER_ID, SEM_CLIENT.CLIENT_ID, SEM_CLIENT.GROUP_ID "
			sql = sql & "From SEM_CLIENT, SEM_COMPUTER "
			sql = sql & "Where SEM_CLIENT.COMPUTER_ID = SEM_COMPUTER.COMPUTER_ID " & IsOUClient & " and SEM_CLIENT.DELETED = 0 and SEM_COMPUTER.DELETED = 0 and SEM_CLIENT.DOMAIN_ID = '" & DomainID & "' "
			Select Case SearchType
				Case 1 'IP groups
					sqlPart1 = "(convert(bigint," & Mid(GroupList(2, x), 1, 3) & ") * 256 * 256 * 256 + "
					sqlPart1 = sqlPart1 & Mid(GroupList(2, x), 4, 3) & " * 256 * 256 + "
					sqlPart1 = sqlPart1 & Mid(GroupList(2, x), 7, 3) & " * 256 + "
					sqlPart1 = sqlPart1 & Mid(GroupList(2, x), 10, 3) & ")"
					sqlPart2 = "(convert(bigint," & Mid(GroupList(3, x), 1, 3) & ") * 256 * 256 * 256 + "
					sqlPart2 = sqlPart2 & Mid(GroupList(3, x), 4, 3) & " * 256 * 256 + "
					sqlPart2 = sqlPart2 & Mid(GroupList(3, x), 7, 3) & " * 256 + "
					sqlPart2 = sqlPart2 & Mid(GroupList(3, x), 10, 3) & ")"
					sql = sql & "and ((SEM_COMPUTER.IP_ADDR1 >= " & sqlPart1 & " and SEM_COMPUTER.IP_ADDR1 <= " & sqlPart2 & ") "
					sql = sql & "or (SEM_COMPUTER.IP_ADDR2 >= " & sqlPart1 & " and SEM_COMPUTER.IP_ADDR2 <= " & sqlPart2 & ") "
					sql = sql & "or (SEM_COMPUTER.IP_ADDR3 >= " & sqlPart1 & " and SEM_COMPUTER.IP_ADDR3 <= " & sqlPart2 & ") "
					sql = sql & "or (SEM_COMPUTER.IP_ADDR4 >= " & sqlPart1 & " and SEM_COMPUTER.IP_ADDR4 <= " & sqlPart2 & ")) "
					sql = sql & "and SEM_CLIENT.POLICY_MODE = 1 "
				Case 2 'Host groups
					If ExactMatch = False then
						sql = sql & "and SEM_COMPUTER.COMPUTER_NAME like '" & replace(replace(GroupList(2, x), "?", "_"), "*", "%") & "' "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 1 "
					Else
						sql = sql & "and SEM_COMPUTER.COMPUTER_NAME = '" & GroupList(2, x) & "' "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 1 "
					End if
				Case 3 'User groups
					If ExactMatch = False Then
						sql = sql & "and SEM_CLIENT.USER_NAME like '" & replace(replace(GroupList(2, x), "?", "_"), "*", "%") & "' "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 0 "
					Else
						sql = sql & "and SEM_CLIENT.USER_NAME = '" & GroupList(2, x) & "' "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 0 "
					End if
				Case 4 'OS groups
					If ExactMatch = False then
						'Add an extra space at the end becasue SEPM has a defect where it adds an extra space in the database
						sql = sql & "and (SEM_COMPUTER.OPERATION_SYSTEM like '" & replace(replace(GroupList(2, x), "?", "_"), "*", "%") & " ' "
						sql = sql & "or SEM_COMPUTER.OPERATION_SYSTEM like '" & replace(replace(GroupList(2, x), "?", "_"), "*", "%") & "') "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 1 "
					Else
						'Add an extra space at the end becasue SEPM has a defect where it adds an extra space in the database
						sql = sql & "and (SEM_COMPUTER.OPERATION_SYSTEM = '" & GroupList(2, x) & " ' "
						sql = sql & "or SEM_COMPUTER.OPERATION_SYSTEM = '" & GroupList(2, x) & "') "
						sql = sql & "and SEM_CLIENT.POLICY_MODE = 1 "
					End if
				Case Else 'Not used
					sql = sql & "and 1 = 2"
			End Select
			If OnlyCheckClientsInTempGroup Then
				sql = sql & "and SEM_CLIENT.group_id = '" & TempGroupID & "' "
			End if
			sql = sql & "Order by SEM_CLIENT.TIME_STAMP DESC;"
			'LogEvent("	::Client List SQL Query: " & sql & vbCRLF)

			Set resultSet = CreateObject("ADODB.recordset")
			resultSet.CursorLocation = 2  'adUseServer
			resultSet.CursorType = 0  'adOpenForwardOnly
			resultSet.LockType = 1  'adLockReadOnly

			Err.Clear      ' Clear the error.
			resultSet.Open sql, connect
			If err.number = 0 Then
				Do While Not resultSet.EOF
					Call MoveClient (GroupList(1, x), resultSet("COMPUTER_NAME") & "", resultSet("USER_NAME") & "", resultSet("COMPUTER_ID"), resultSet("CLIENT_ID"), resultSet("GROUP_ID"))
					GroupList(4, x) = True
					resultSet.MoveNext
				Loop
			Else
				LogEvent("		::Failed to retrieve client list from database." & vbCRLF)
			End If

			resultSet.Close
		End if
	Next
	If not FoundClients then LogEvent("		::No clients found that need to be moved." & vbCRLF)
End Sub


Sub MoveClient (GroupName, ComputerName, UserName, ComputerID, ClientID, ComputerGroupID)
	Dim sql, SQLPart, resultSet
	Dim GroupID
	
	On Error Resume Next
	Err.Clear      ' Clear the error.

	FoundClients = True

	GroupID = ""
	'SQL 2005/2008 select statement
	sql = "Select IDENTITY_MAP.ID from IDENTITY_MAP, BASIC_METADATA where IDENTITY_MAP.Name like '" & RootGroup & "\" & GroupName & "' and IDENTITY_MAP.Type = 'SemClientGroup' and BASIC_METADATA.type = 'SemClientGroupTree' and CONVERT(varchar(max),CONVERT(varbinary(max), BASIC_METADATA.CONTENT)) like '%id=" & chr(34) & "' + IDENTITY_MAP.ID + '" & chr(34) & "%' and IDENTITY_MAP.DOMAIN_ID = '" & DomainID & "' and BASIC_METADATA.DOMAIN_ID = '" & DomainID & "';"
	Set resultSet = connect.Execute(sql)
	'Sybase select statement
	sql = "Select IDENTITY_MAP.ID from IDENTITY_MAP, BASIC_METADATA where IDENTITY_MAP.Name like '" & RootGroup & "\" & GroupName & "' and IDENTITY_MAP.Type = 'SemClientGroup' and BASIC_METADATA.type = 'SemClientGroupTree' and CONVERT(text, BASIC_METADATA.CONTENT) like '%id=" & chr(34) & "' + IDENTITY_MAP.ID + '" & chr(34) & "%' and IDENTITY_MAP.DOMAIN_ID = '" & DomainID & "' and BASIC_METADATA.DOMAIN_ID = '" & DomainID & "';"
	Set resultSet = connect.Execute(sql)
	GroupID = resultSet("ID") & ""
	
	'SQL 2000:  This also works with all other DBs, as long as it can get the full SemClientGroupTree metadata
	If trim(GroupID) = "" then
		sql = "Select ID from IDENTITY_MAP where NAME like '" & RootGroup & "\" & GroupName & "' and TYPE = 'SemClientGroup' and DOMAIN_ID = '" & DomainID & "';"
		Set resultSet = connect.Execute(sql)
		Do While NOT resultSet.EOF
			If NOT IsNull(resultSet("ID")) then
				If InStr(GroupStructureXML, lcase("id=" & chr(34) & resultSet("ID") & chr(34))) > 0 then GroupID = resultSet("ID")
			End if
			resultSet.MoveNext
		Loop
	End if
	resultSet.Close

	If trim(GroupID) = "" then
		LogEvent("		::Failed to move Client '" & ComputerName  & "' User '" & UserName  & "'.  The new group is invalid." & vbCRLF)
		AnyFailures = True
	ElseIf trim(GroupID) = trim(ComputerGroupID) Then
		LogEvent("		::Skipping Client '" & ComputerName  & "' User '" & UserName  & "'.  Client already in the correct group." & vbCRLF)
	Else
		Err.Clear      ' Clear the error.
		
		LogEvent(Date() & " " & Time() & "	::Moving Client '" & ComputerName  & "' User '" & UserName  & "' to group: '" & RootGroup & "\" & GroupName & "' ID: " & GroupID & vbCRLF)
		If LogOldClientGroup = True then LogEvent("			::Original group was: '" & GetOldGroupName(ComputerGroupID) & "' ID: " & ComputerGroupID & vbCRLF)

		connectUpdate.BeginTrans
		If SearchType <> 3 then
			sql = "Update SEM_Agent set GROUP_ID='" & GroupID & "', TIME_STAMP=" & NewTimeStamp & ", USN=" & NewUSN & " where COMPUTER_ID='" & ComputerID & "';"
			connectUpdate.Execute sql
		End if
		SQLPart = "Select Case When (Select Sum(A.GROUP_IS_OU) + Sum(A.PIN_MARK) from SEM_Client as A, SEM_Client as B where A.POLICY_MODE = 0 and A.USER_NAME = B.USER_NAME and A.USER_DOMAIN_NAME = B.USER_DOMAIN_NAME and B.CLIENT_ID='" & ClientID & "') > 0 then 1 Else 0 End"
		SQLPart = "(Case When (Select Sum(GROUP_IS_OU) + Sum(PIN_MARK) + (" & SQLPart & ") from SEM_Client where CLIENT_ID='" & ClientID & "') > 0 then 1 Else 0 End)"
		sql = "Update SEM_Client set PIN_MARK=" & SQLPart & ", GROUP_ID='" & GroupID & "', TIME_STAMP=" & NewTimeStamp & ", USN=" & NewUSN & ", GROUP_IS_OU=0 where CLIENT_ID='" & ClientID & "';"
		connectUpdate.Execute sql
		If err.number <> 0 Then
			LogEvent("				::Failed to move client error " & err.number & vbCRLF)
			Err.Clear      ' Clear the error.
			connectUpdate.RollbackTrans
			If err.number <> 0 Then LogEvent("				::Failed to rollback transaction " & err.number & vbCRLF)
			AnyFailures = True
		Else
			UpdateFlag = true
			connectUpdate.CommitTrans
			If err.number <> 0 Then
				LogEvent("				::Failed to commit transaction " & err.number & vbCRLF)
				AnyFailures = True
			End if
		End if
	End if
End Sub


Function GetOldGroupName (ComputerGroupID)
	Dim sql, resultSet
	
	On Error Resume Next	
	GetOldGroupName = "N/A"
	
	sql = "Select Name from IDENTITY_MAP where ID = '" & ComputerGroupID & "';"
	'LogEvent("	::Get group name SQL Query: " & sql & vbCRLF)
	
	Set resultSet = connect.Execute(sql)
	
	If NOT resultSet.EOF then
		GetOldGroupName = resultSet("Name")
	End if
	
	resultSet.Close
End Function


Function GetSubnet (SubnetText)
	Dim IPPart, NewSubnet
	IPPart = Split(SubnetText, ".")
	GetSubnet = "000000000000"	
	If Ubound(IPPart) = 3 then
		NewSubnet = right("00" & IPPart(0), 3) & right("00" & IPPart(1), 3) & right("00" & IPPart(2), 3) & right("00" & IPPart(3), 3)
		If IsNumeric(NewSubnet) then GetSubnet = NewSubnet
	End if
End Function


Function GetSubnetMasked (IPAddressNum, SubnetNote, IsLow)
	Dim x, MyNum
	GetSubnetMasked = "000000000000"
	If IsNumeric(IPAddressNum) and IsNumeric(SubnetNote) and len(IPAddressNum) = 12 and len(SubnetNote) = 12 then
		GetSubnetMasked = ""
		For x = 0 to 3
			If IsLow then
				MyNum = CInt(Mid(IPAddressNum, x * 3 + 1, 3)) and CInt(Mid(SubnetNote, x * 3 + 1, 3))
			Else
				MyNum = CInt(Mid(IPAddressNum, x * 3 + 1, 3)) or (CInt(Mid(SubnetNote, x * 3 + 1, 3)) xOr 255)
			End if
			GetSubnetMasked = GetSubnetMasked & right("00" & MyNum, 3)
		Next
	End if
End Function


Function GetSubnetNote (SubnetNote)
	GetSubnetNote = "000000000000"
	If IsNumeric(SubnetNote) then
		GetSubnetNote = Left(Left("255255255255", (CInt(SubnetNote) \ 8) * 3) & Right("00" & CStr(255 xOr (2^(8 - (SubnetNote Mod 8)) - 1)), 3) & "000000000000", 12)
	End if
End Function


Function GetGroupNameIP (strText1, strText2, strText3, strText4)
	Dim x, y
	Dim IPAddress(3)
	GetGroupNameIP = ""
	IPAddress(0) = GetSubnet(strText1)
	IPAddress(1) = GetSubnet(strText2)
	IPAddress(2) = GetSubnet(strText3)
	IPAddress(3) = GetSubnet(strText4)
	For x = 1 to ListCount
		If GroupList(0, x) = DomainNum then
			For y = 0 to 3
				If (IPAddress(y) >= GroupList(2, x)) and (IPAddress(y) <= GroupList(3, x)) and (IPAddress(y) > 0) then
					GetGroupNameIP = GroupList(1, x)
					Exit Function
				End if
			Next
		End if
	Next
End Function


Function GetGroupName (strText)
	Dim x, IPAddress
	Dim newStrText
	newStrText = lcase(strText)
	newStrText = replace(newStrText, "'", "''")
	GetGroupName = ""
	If SearchType = 4 or AllClientNamesWild = True or RegExTest("*" & newStrText & "*", AllClientNames) > 0 then
		For x = 1 to ListCount
			If GroupList(0, x) = DomainNum then
				If (RegExComp(GroupList(2, x), newStrText) > 0) then
					GetGroupName = GroupList(1, x)
					GroupList(4, x) = True
					Exit Function
				End if
			End if
		Next
	End if
End Function


Function RegExTest (patrn, strng)
	Dim regEx, Match, Matches
	Set regEx = New RegExp
	RegExTest = 0
	Dim newPatrn
	newPatrn = patrn
	newPatrn = replace(newPatrn, "\", "\\")
	newPatrn = replace(newPatrn, "^", "\^")
	newPatrn = replace(newPatrn, "$", "\$")
	newPatrn = replace(newPatrn, "+", "\+")
	newPatrn = replace(newPatrn, "|", "\|")
	newPatrn = replace(newPatrn, "{", "\{")
	newPatrn = replace(newPatrn, "}", "\}")
	newPatrn = replace(newPatrn, "[", "\[")
	newPatrn = replace(newPatrn, "]", "\]")
	newPatrn = replace(newPatrn, "(", "\)")
	newPatrn = replace(newPatrn, ".", "\.")
	newPatrn = replace(newPatrn, "*", ".{0,}")
	newPatrn = replace(newPatrn, "?", ".{1}")
	regEx.Pattern = newPatrn
	regEx.IgnoreCase = True
	regEx.Global = True
	Set Matches = regEx.Execute(strng)
	RegExTest = Matches.Count
End Function


Function RegExComp (patrn, strng)
	Dim regEx, Match, Matches
	Set regEx = New RegExp
	RegExComp = 0
	If (InStr(patrn, "?") > 0) or (InStr(patrn, "*") > 0) then
		Dim newPatrn
		newPatrn = patrn
		newPatrn = replace(newPatrn, "\", "\\")
		newPatrn = replace(newPatrn, "^", "\^")
		newPatrn = replace(newPatrn, "$", "\$")
		newPatrn = replace(newPatrn, "+", "\+")
		newPatrn = replace(newPatrn, "|", "\|")
		newPatrn = replace(newPatrn, "{", "\{")
		newPatrn = replace(newPatrn, "}", "\}")
		newPatrn = replace(newPatrn, "[", "\[")
		newPatrn = replace(newPatrn, "]", "\]")
		newPatrn = replace(newPatrn, "(", "\)")
		newPatrn = replace(newPatrn, ".", "\.")
		newPatrn = replace(newPatrn, "*", ".{0,}")
		newPatrn = replace(newPatrn, "?", ".{1}")
		regEx.Pattern = "^" & newPatrn & "$"
		regEx.IgnoreCase = True
		regEx.Global = True
		Set Matches = regEx.Execute(strng)
		RegExComp = Matches.Count
	Else
		If patrn = strng then
			RegExComp = 1
		Else
			RegExComp = 0
		End if
	End if
End Function


Sub ShowMessage (strng)
	If not SilentMode then MsgBox(strng)
End Sub


Sub LogEvent (strng)
	If EnableLogging then OutputFile.Write(strng)
End Sub


Sub CloseLog ()
	On Error Resume Next
	Err.Clear      ' Clear the error.
	
	If UpdateFlag then
		Dim sql, resultSet
		sql = "Update SE_Global set SEQ_NUM = (select SEQ_NUM + 1 as NewUSN from SE_Global);"
		'LogEvent("	::SetUSN query: " & sql & vbCRLF)
		Set resultSet = connect.Execute(sql)
		If err.number <> 0 Then LogEvent("	::Failed to update USN number." & vbCRLF)
	End if
	
	'End output
	If AnyFailures = True then
		LogEvent("########################################################################" & vbCRLF & "# MoveClient Ended with Failures		" & Date() & " " & Time() & vbCRLF & "########################################################################" & vbCRLF)
	Else
		LogEvent("########################################################################" & vbCRLF & "# End MoveClient		" & Date() & " " & Time() & vbCRLF & "########################################################################" & vbCRLF)
	End if
	
	If EnableLogging then OutputFile.Close
	connect.Close
	Set connect = Nothing
	WScript.Quit(0)
End Sub


Function RSBinaryToString(xBinary)
	'Antonin Foller, http://www.motobit.com
	'RSBinaryToString converts binary data (VT_UI1 | VT_ARRAY Or MultiByte string)
	'to a string (BSTR) using ADO recordset

	Dim Binary
	'MultiByte data must be converted To VT_UI1 | VT_ARRAY first.
	If vartype(xBinary) = 8 Then Binary = MultiByteToBinary(xBinary) Else Binary = xBinary

	Dim RS, LBinary
	Const adLongVarChar = 201
	Set RS = CreateObject("ADODB.Recordset")
	LBinary = LenB(Binary)

	If LBinary > 0 Then
		RS.Fields.Append "mBinary", adLongVarChar, LBinary
		RS.Open
		RS.AddNew
		RS("mBinary").AppendChunk Binary 
		RS.Update
		RSBinaryToString = RS("mBinary")
	Else
		RSBinaryToString = ""
	End If
End Function


' ------ Call MD5 Hash Function --------
' --------------------------------------

' --------------------------------------
Public Function MD5FileHash(strFile)

	Dim strMD5 : strMD5 = ""
	Dim ofso : Set ofso = CreateObject("Scripting.FileSystemObject")

	If ofso.FileExists(strFile) then

		strMD5 = BinaryToString(ReadTextFile(strFile, ""))

		MD5FileHash = CalculateMD5(strMD5)

	Else

		MD5FileHash = strFile & VbCrLf & "Error: File not found"

	End if

End Function

' --------------------------------------
Function ReadTextFile(FileName, CharSet)

	Const adTypeText = 2
	Dim BinaryStream : Set BinaryStream = CreateObject("ADODB.Stream")

	BinaryStream.Type = adTypeText

	If Len(CharSet) > 0 Then

		BinaryStream.CharSet = CharSet

	End If

	BinaryStream.Open
	BinaryStream.LoadFromFile FileName

	ReadTextFile = BinaryStream.ReadText
	
End Function

' --------------------------------------
Function BinaryToString(Binary)

	Dim cl1, cl2, cl3, pl1, pl2, pl3
	Dim L
	cl1 = 1
	cl2 = 1
	cl3 = 1
	L = LenB(Binary)

	Do While cl1<=L

		pl3 = pl3 & Chr(AscB(MidB(Binary,cl1,1)))
		cl1 = cl1 + 1
		cl3 = cl3 + 1

		If cl3>300 Then
			pl2 = pl2 & pl3
			pl3 = ""
			cl3 = 1
			cl2 = cl2 + 1

			If cl2>200 Then

				pl1 = pl1 & pl2
				pl2 = ""
				cl2 = 1

			End If
		End If
	Loop

	BinaryToString = pl1 & pl2 & pl3
	
End Function

' --------------------------------------
Private Function MD5Round(strRound, a, b, C, d, X, S, ac)

	Select Case strRound

		Case "FF"
			a = MD5LongAdd4(a, (b And C) Or (Not (b) And d), X, ac)
			a = MD5Rotate(a, S)
			a = MD5LongAdd(a, b)
        
		Case "GG"
			a = MD5LongAdd4(a, (b And d) Or (C And Not (d)), X, ac)
			a = MD5Rotate(a, S)
			a = MD5LongAdd(a, b)
            
		Case "HH"
			a = MD5LongAdd4(a, b Xor C Xor d, X, ac)
			a = MD5Rotate(a, S)
			a = MD5LongAdd(a, b)
            
		Case "II"
			a = MD5LongAdd4(a, C Xor (b Or Not (d)), X, ac)
			a = MD5Rotate(a, S)
			a = MD5LongAdd(a, b)
	End Select

End Function

' --------------------------------------
Private Function MD5Rotate(lngValue, lngBits)

	Dim lngSign
	Dim lngI
    
	lngBits = (lngBits Mod 32)

	If lngBits = 0 Then MD5Rotate = lngValue: Exit Function

	For lngI = 1 To lngBits
		lngSign = lngValue And &HC0000000
		lngValue = (lngValue And &H3FFFFFFF) * 2
		lngValue = lngValue Or ((lngSign < 0) And 1) Or (CBool(lngSign And &H40000000) And &H80000000)
	Next

	MD5Rotate = lngValue

End Function

' --------------------------------------
Private Function TRID()

	Dim sngNum, lngnum
	Dim strResult

	sngNum = Rnd(2147483648)
	strResult = CStr(sngNum)

	strResult = Replace(strResult, "0.", "")
	strResult = Replace(strResult, ".", "")
	strResult = Replace(strResult, "E-", "")

	TRID = strResult

End Function

' --------------------------------------
Private Function MD564Split(lngLength, bytBuffer())

	Dim lngBytesTotal, lngBytesToAdd
	Dim intLoop, intLoop2, lngTrace
	Dim intInnerLoop, intLoop3

	lngBytesTotal = lngTrack Mod 64
	lngBytesToAdd = 64 - lngBytesTotal
	lngTrack = (lngTrack + lngLength)

	If lngLength >= lngBytesToAdd Then

		For intLoop = 0 To lngBytesToAdd - 1
			arrSplit64(lngBytesTotal + intLoop) = bytBuffer(intLoop)
		Next

		MD5Conversion arrSplit64

		lngTrace = (lngLength) Mod 64

		For intLoop2 = lngBytesToAdd To lngLength - intLoop - lngTrace Step 64

			For intInnerLoop = 0 To 63
				arrSplit64(intInnerLoop) = bytBuffer(intLoop2 + intInnerLoop)
			Next

		MD5Conversion arrSplit64

		Next

		lngBytesTotal = 0
	Else

		intLoop2 = 0

	End If

	For intLoop3 = 0 To lngLength - intLoop2 - 1

		arrSplit64(lngBytesTotal + intLoop3) = bytBuffer(intLoop2 + intLoop3)

	Next

End Function

' --------------------------------------
Private Function MD5StringArray(strInput)

	Dim intLoop
	Dim bytBuffer()
	ReDim bytBuffer(Len(strInput))
    
	For intLoop = 0 To Len(strInput) - 1
		bytBuffer(intLoop) = Asc(Mid(strInput, intLoop + 1, 1))
	Next

	MD5StringArray = bytBuffer

End Function

' --------------------------------------
Private Sub MD5Conversion(bytBuffer())

	Dim X(16), a
	Dim b, C
	Dim d

	a = arrLongConversion(1)
	b = arrLongConversion(2)
	C = arrLongConversion(3)
	d = arrLongConversion(4)

	MD5Decode 64, X, bytBuffer

	MD5Round "FF", a, b, C, d, X(0), S11, -680876936
	MD5Round "FF", d, a, b, C, X(1), S12, -389564586
	MD5Round "FF", C, d, a, b, X(2), S13, 606105819
	MD5Round "FF", b, C, d, a, X(3), S14, -1044525330
	MD5Round "FF", a, b, C, d, X(4), S11, -176418897
	MD5Round "FF", d, a, b, C, X(5), S12, 1200080426
	MD5Round "FF", C, d, a, b, X(6), S13, -1473231341
	MD5Round "FF", b, C, d, a, X(7), S14, -45705983
	MD5Round "FF", a, b, C, d, X(8), S11, 1770035416
	MD5Round "FF", d, a, b, C, X(9), S12, -1958414417
	MD5Round "FF", C, d, a, b, X(10), S13, -42063
	MD5Round "FF", b, C, d, a, X(11), S14, -1990404162
	MD5Round "FF", a, b, C, d, X(12), S11, 1804603682
	MD5Round "FF", d, a, b, C, X(13), S12, -40341101
	MD5Round "FF", C, d, a, b, X(14), S13, -1502002290
	MD5Round "FF", b, C, d, a, X(15), S14, 1236535329

	MD5Round "GG", a, b, C, d, X(1), S21, -165796510
	MD5Round "GG", d, a, b, C, X(6), S22, -1069501632
	MD5Round "GG", C, d, a, b, X(11), S23, 643717713
	MD5Round "GG", b, C, d, a, X(0), S24, -373897302
	MD5Round "GG", a, b, C, d, X(5), S21, -701558691
	MD5Round "GG", d, a, b, C, X(10), S22, 38016083
	MD5Round "GG", C, d, a, b, X(15), S23, -660478335
	MD5Round "GG", b, C, d, a, X(4), S24, -405537848
	MD5Round "GG", a, b, C, d, X(9), S21, 568446438
	MD5Round "GG", d, a, b, C, X(14), S22, -1019803690
	MD5Round "GG", C, d, a, b, X(3), S23, -187363961
	MD5Round "GG", b, C, d, a, X(8), S24, 1163531501
	MD5Round "GG", a, b, C, d, X(13), S21, -1444681467
	MD5Round "GG", d, a, b, C, X(2), S22, -51403784
	MD5Round "GG", C, d, a, b, X(7), S23, 1735328473
	MD5Round "GG", b, C, d, a, X(12), S24, -1926607734

	MD5Round "HH", a, b, C, d, X(5), S31, -378558
	MD5Round "HH", d, a, b, C, X(8), S32, -2022574463
	MD5Round "HH", C, d, a, b, X(11), S33, 1839030562
	MD5Round "HH", b, C, d, a, X(14), S34, -35309556
	MD5Round "HH", a, b, C, d, X(1), S31, -1530992060
	MD5Round "HH", d, a, b, C, X(4), S32, 1272893353
	MD5Round "HH", C, d, a, b, X(7), S33, -155497632
	MD5Round "HH", b, C, d, a, X(10), S34, -1094730640
	MD5Round "HH", a, b, C, d, X(13), S31, 681279174
	MD5Round "HH", d, a, b, C, X(0), S32, -358537222
	MD5Round "HH", C, d, a, b, X(3), S33, -722521979
	MD5Round "HH", b, C, d, a, X(6), S34, 76029189
	MD5Round "HH", a, b, C, d, X(9), S31, -640364487
	MD5Round "HH", d, a, b, C, X(12), S32, -421815835
	MD5Round "HH", C, d, a, b, X(15), S33, 530742520
	MD5Round "HH", b, C, d, a, X(2), S34, -995338651

	MD5Round "II", a, b, C, d, X(0), S41, -198630844
	MD5Round "II", d, a, b, C, X(7), S42, 1126891415
	MD5Round "II", C, d, a, b, X(14), S43, -1416354905
	MD5Round "II", b, C, d, a, X(5), S44, -57434055
	MD5Round "II", a, b, C, d, X(12), S41, 1700485571
	MD5Round "II", d, a, b, C, X(3), S42, -1894986606
	MD5Round "II", C, d, a, b, X(10), S43, -1051523
	MD5Round "II", b, C, d, a, X(1), S44, -2054922799
	MD5Round "II", a, b, C, d, X(8), S41, 1873313359
	MD5Round "II", d, a, b, C, X(15), S42, -30611744
	MD5Round "II", C, d, a, b, X(6), S43, -1560198380
	MD5Round "II", b, C, d, a, X(13), S44, 1309151649
	MD5Round "II", a, b, C, d, X(4), S41, -145523070
	MD5Round "II", d, a, b, C, X(11), S42, -1120210379
	MD5Round "II", C, d, a, b, X(2), S43, 718787259
	MD5Round "II", b, C, d, a, X(9), S44, -343485551

	arrLongConversion(1) = MD5LongAdd(arrLongConversion(1), a)
	arrLongConversion(2) = MD5LongAdd(arrLongConversion(2), b)
	arrLongConversion(3) = MD5LongAdd(arrLongConversion(3), C)
	arrLongConversion(4) = MD5LongAdd(arrLongConversion(4), d)

End Sub

' --------------------------------------
Private Function MD5LongAdd(lngVal1, lngVal2)

	Dim lngHighWord
	Dim lngLowWord
	Dim lngOverflow

	lngLowWord = (lngVal1 And &HFFFF&) + (lngVal2 And &HFFFF&)
	lngOverflow = lngLowWord \ 65536
	lngHighWord = (((lngVal1 And &HFFFF0000) \ 65536) + ((lngVal2 And &HFFFF0000) \ 65536) + lngOverflow) And &HFFFF&

	MD5LongAdd = MD5LongConversion((lngHighWord * 65536) + (lngLowWord And &HFFFF&))

End Function

' --------------------------------------
Private Function MD5LongAdd4(lngVal1, lngVal2, lngVal3, lngVal4)

	Dim lngHighWord
	Dim lngLowWord
	Dim lngOverflow

	lngLowWord = (lngVal1 And &HFFFF&) + (lngVal2 And &HFFFF&) + (lngVal3 And &HFFFF&) + (lngVal4 And &HFFFF&)
	lngOverflow = lngLowWord \ 65536
	lngHighWord = (((lngVal1 And &HFFFF0000) \ 65536) + ((lngVal2 And &HFFFF0000) \ 65536) + ((lngVal3 And &HFFFF0000) \ 65536) + ((lngVal4 And &HFFFF0000) \ 65536) + lngOverflow) And &HFFFF&
	MD5LongAdd4 = MD5LongConversion((lngHighWord * 65536) + (lngLowWord And &HFFFF&))

End Function

' --------------------------------------
Private Sub MD5Decode(intLength, lngOutBuffer(), bytInBuffer())

	Dim intDblIndex
	Dim intByteIndex
	Dim dblSum

	intDblIndex = 0
    
	For intByteIndex = 0 To intLength - 1 Step 4

		dblSum = bytInBuffer(intByteIndex) + bytInBuffer(intByteIndex + 1) * 256 + bytInBuffer(intByteIndex + 2) * 65536 + bytInBuffer(intByteIndex + 3) * 16777216
		lngOutBuffer(intDblIndex) = MD5LongConversion(dblSum)
		intDblIndex = (intDblIndex + 1)

	Next

End Sub

' --------------------------------------
Private Function MD5LongConversion(dblValue)

	If dblValue < 0 Or dblValue >= OFFSET_4 Then Error 6

	If dblValue <= MAXINT_4 Then
		MD5LongConversion = dblValue
	Else
		MD5LongConversion = dblValue - OFFSET_4
	End If

End Function

' --------------------------------------
Private Sub MD5Finish()

	Dim dblBits
	Dim arrPadding(72)
	Dim lngBytesBuffered

	arrPadding(0) = &H80

	dblBits = lngTrack * 8

	lngBytesBuffered = lngTrack Mod 64
    
	If lngBytesBuffered <= 56 Then
		MD564Split (56 - lngBytesBuffered), arrPadding
	Else
		MD564Split (120 - lngTrack), arrPadding
	End If

	arrPadding(0) = MD5LongConversion(dblBits) And &HFF&
	arrPadding(1) = MD5LongConversion(dblBits) \ 256 And &HFF&
	arrPadding(2) = MD5LongConversion(dblBits) \ 65536 And &HFF&
	arrPadding(3) = MD5LongConversion(dblBits) \ 16777216 And &HFF&
	arrPadding(4) = 0
	arrPadding(5) = 0
	arrPadding(6) = 0
	arrPadding(7) = 0

	MD564Split 8, arrPadding

End Sub

' --------------------------------------
Private Function MD5StringChange(lngnum)

	Dim bytA
	Dim bytB
	Dim bytC
	Dim bytD

	bytA = lngnum And &HFF&
	If bytA < 16 Then
		MD5StringChange = "0" & Hex(bytA)
	Else
		MD5StringChange = Hex(bytA)
	End If

	bytB = (lngnum And &HFF00&) \ 256
	If bytB < 16 Then
		MD5StringChange = MD5StringChange & "0" & Hex(bytB)
	Else
		MD5StringChange = MD5StringChange & Hex(bytB)
	End If

	bytC = (lngnum And &HFF0000) \ 65536
	If bytC < 16 Then
		MD5StringChange = MD5StringChange & "0" & Hex(bytC)
	Else
		MD5StringChange = MD5StringChange & Hex(bytC)
	End If

	If lngnum < 0 Then
		bytD = ((lngnum And &H7F000000) \ 16777216) Or &H80&
	Else
		bytD = (lngnum And &HFF000000) \ 16777216
	End If

	If bytD < 16 Then
		MD5StringChange = MD5StringChange & "0" & Hex(bytD)
	Else
		MD5StringChange = MD5StringChange & Hex(bytD)
	End If

End Function

' --------------------------------------
Private Function MD5Value()

	MD5Value = LCase(MD5StringChange(arrLongConversion(1)) & MD5StringChange(arrLongConversion(2)) & MD5StringChange(arrLongConversion(3)) & MD5StringChange(arrLongConversion(4)))

End Function

' --------------------------------------
Public Function CalculateMD5(strMessage)

	Dim bytBuffer

	bytBuffer = MD5StringArray(strMessage)

	MD5Start

		MD564Split Len(strMessage), bytBuffer

	MD5Finish

	CalculateMD5 = MD5Value

End Function

' --------------------------------------
Private Sub MD5Start()

	lngTrack = 0
	arrLongConversion(1) = MD5LongConversion(1732584193)
	arrLongConversion(2) = MD5LongConversion(4023233417)
	arrLongConversion(3) = MD5LongConversion(2562383102)
	arrLongConversion(4) = MD5LongConversion(271733878)

End Sub
